﻿using DerpingDrivers.Exceptions;
using Microsoft.Win32;
using RunProcessAsTask;

namespace DerpingDrivers.Util
{
    public static class BcdHelper
    {
        public static bool IsTestSignEnabled
        {
            get
            {
                var bootmgrDefaultGuid = (string) Registry.GetValue(
                    @"HKEY_LOCAL_MACHINE\BCD00000000\Objects\{9DEA862C-5CDD-4E70-ACC1-F32B344D4795}\Elements\23000003",
                    "Element", null);

                var allowPrereleaseSignatures = (byte[]) Registry.GetValue(
                    $@"HKEY_LOCAL_MACHINE\BCD00000000\Objects\{bootmgrDefaultGuid}\Elements\16000049",
                    "Element", default(byte[]));

                return allowPrereleaseSignatures != null && bool.Parse(allowPrereleaseSignatures[0].ToString());
            }
        }

        public static void EnableTestSigning()
        {
            // NOTE: haven't found an API for this (yet), so system() we go!
            var ret = ProcessEx.RunAsync("Bcdedit.exe", "-set TESTSIGNING ON").Result;

            if (ret.ExitCode != 0)
            {
                throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't enable TESTSIGNING");
            }
        }

        public static void DisableTestSigning()
        {
            // NOTE: haven't found an API for this (yet), so system() we go!
            var ret = ProcessEx.RunAsync("Bcdedit.exe", "-set TESTSIGNING OFF").Result;

            if (ret.ExitCode != 0)
            {
                throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't disable TESTSIGNING");
            }
        }
    }
}